'use strict';

var _yeomanGenerator = require('yeoman-generator');

var _yeomanGenerator2 = _interopRequireDefault(_yeomanGenerator);

var _chalk = require('chalk');

var _chalk2 = _interopRequireDefault(_chalk);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _process = require('process');

var _process2 = _interopRequireDefault(_process);

var _rimraf = require('rimraf');

var _rimraf2 = _interopRequireDefault(_rimraf);

var _underscore = require('underscore.string');

var _underscore2 = _interopRequireDefault(_underscore);

var _yosay = require('yosay');

var _yosay2 = _interopRequireDefault(_yosay);

var _helpers = require('./helpers');

var _config = require('./config');

var _config2 = _interopRequireDefault(_config);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; } /* tslint:disable no-require-imports no-var-requires adjacent-overload-signatures max-line-length no-empty ordered-imports */


let remote = require('yeoman-remote');

// config


// yell('test');

class PrebootGenerator extends _yeomanGenerator2.default {

  constructor(args, options) {
    super(args, options);

    this.argument('appname', {
      desc: 'The name of the application (e.g. \'example\')',
      type: String,
      optional: true,
      default: _path2.default.basename(_process2.default.cwd())
    });
  }

  // this is the initializer method of the generator
  get initializing() {

    return {
      cleaning: function cleaning() {
        // async
        let done = this.async();

        if (!this.options['cache']) {
          // new counter
          let counter = (0, _helpers.yell)('Cleaning cache ...');
          counter.start();

          // we would the defaults here
          (0, _rimraf2.default)(remote.cacheRoot(), () => {
            counter.stop();
            done();
          });
        } else {
          done();
        }
      },
      hello: function hello() {
        // say yo!
        this.log((0, _yosay2.default)('Greetings! Preboot your next Angular 2 project'));
      }
    };
  }

  get prompting() {
    return {
      appName: function appName() {
        var _this = this;

        return _asyncToGenerator(function* () {
          try {
            let prompt = [{
              type: 'input',
              name: 'app',
              message: 'What is the name of your fun new app?',
              default: _this.appname,
              store: true
            }];

            var _ref = yield _this.prompt(prompt);

            let app = _ref.app;

            _this.options.appname = _underscore2.default.camelize(_underscore2.default.slugify(_underscore2.default.humanize(app)));
          } catch (err) {
            _this.error('Error: ' + err.message);
          }
        })();
      },
      gitUserName: function gitUserName() {
        var _this2 = this;

        return _asyncToGenerator(function* () {
          try {
            let prompt = [{
              type: 'input',
              name: 'name',
              message: 'What is your name?',
              default: _this2.user.git.name(),
              store: true
            }];

            var _ref2 = yield _this2.prompt(prompt);

            let name = _ref2.name;

            _this2.options.name = name;
          } catch (err) {
            _this2.error('Error: ' + err.message);
          }
        })();
      },
      gitUserEmail: function gitUserEmail() {
        var _this3 = this;

        return _asyncToGenerator(function* () {
          try {
            let prompt = [{
              type: 'input',
              name: 'email',
              message: 'What is your email?',
              default: _this3.user.git.email(),
              store: true
            }];

            var _ref3 = yield _this3.prompt(prompt);

            let email = _ref3.email;

            _this3.options.email = email;
          } catch (err) {
            _this3.error('Error: ' + err.message);
          }
        })();
      },
      yarn: function yarn() {
        var _this4 = this;

        return _asyncToGenerator(function* () {
          try {
            let prompt = [{
              type: 'confirm',
              name: 'yarn',
              message: 'Would you like to use yarn?',
              default: false,
              store: true
            }];

            var _ref4 = yield _this4.prompt(prompt);

            let yarn = _ref4.yarn;

            _this4.options.yarn = yarn;
          } catch (err) {
            _this4.error('Error: ' + err.message);
          }
        })();
      }
    };
  }

  get configuring() {
    return {};
  }

  get writing() {
    return {
      staging: function staging() {
        let done = this.async();
        // new counter
        let counter = (0, _helpers.yell)('Staging Angular 2 Preboot ...');
        counter.start();

        remote(_config2.default.tar, (err, cached) => {

          if (err) {
            this.error('Error ' + err);
          }

          // we have a different root for the sources
          this.sourceRoot(_path2.default.join(cached));

          counter.stop();

          this.fs.copy(this.templatePath('**/*'), this.destinationPath(''), {
            globOptions: {
              dot: true
            }
          });

          this.fs.extendJSON(this.destinationPath('package.json'), {
            author: {
              name: this.options.name,
              email: this.options.email
            },
            bugs: {
              url: ''
            },
            description: this.options.description,
            homepage: '',
            repository: {
              type: 'git',
              url: ''
            },
            version: '0.0.1'
          });

          done();
        });
      },
      npm: function npm() {

        // npm
        if (!this.options['skip-install']) {
          // new counter
          let cl = console.log;
          console.log = () => {};

          let counter = (0, _helpers.yell)('Installing dependencies via ' + (this.options.yarn ? 'yarn' : 'npm') + ' ...');
          counter.start();

          this.runInstall(this.options.yarn ? 'yarn' : 'npm', '', this.options.yarn ? {} : _config2.default.npm, () => {
            console.log = cl;
            counter.stop();
          });
        } else {
          this.log('\nPlease run ' + _chalk2.default.yellow.bold('npm install') + '.\n            \nAfterwards run ' + _chalk2.default.yellow.bold('npm start'));
        }
      }
    };
  }

}

// exporting generator as CommonJS module
module.exports = PrebootGenerator;